#ifndef __SP_DIALOG_H
#define __SP_DIALOG_H

#include <qdialog.h>
#include <qpushbutton.h>
#include <qlabel.h>
#include <qcombo.h>
#include <qvalidator.h>

#include "defines.h"
#include "mylineedit.h"

struct SOptions
{
   float timeAfterKill;
   int playerKey[2][PlayerKeyNum];
   int functionKey[FunctionKeyNum];
   unsigned refreshTime,startHitPoints[2],lastConfig;
};

struct SConfig
{
   float gamespeed,gravity,acc,energyNeed,sunEnergy,rotationSpeed,
      mineActivateTime,mineFuel,shotSpeed,shotEnergyNeed,mineEnergyNeed,
      rotationEnergyNeed,startPosX,startPosY,startVelX,startVelY;
   unsigned bulletDamage,shipDamage,mineDamage,maxBullets,maxMines;
};
#define predefinedConfigNum 4
const SConfig predefinedConfig[]={{1.0, 2200.0, 0.2, 1.0, 9000.0, 1.0,
                                    15.0, 65.0, 3.0, 10.0, 5.0,
                                    0.2, -130.0, -100.0, 3.0, -1.7,
                                    20, 50, 30, 5, 3},
                                   {1.0, 2200.0, 0.2, 1.0, 9000.0, 1.0,
                                    15.0, 40.0, 5.0, 20.0, 10.0,
                                    0.2, -50.0, -150.0, 3.5, 0.9,
                                    20, 50, 30, 6, 2},
                                   {1.3, 2200.0, 0.2, 1.0, 13000.0, 1.0,
                                    15.0, 50.0, 4.0, 10.0, 10.0,
                                    0.2, -50.0, -150.0, 3.2, -0.9,
                                    20, 50, 30, 7, 5},
                                   {1.0, 2200.0, 0.2, 1.0, 9000.0, 1.0,
                                    15.0, 40.0, 5.0, 60.0, 50.0,
                                    0.4, -50.0, -170.0, 3.0, -0.5,
                                    20, 50, 30, 5, 3}};
const char predefinedConfigName[predefinedConfigNum][15]=
{"Default","Bullet","Chaos","Lack of energy"}; 




class KeySetup:public QDialog
{
   Q_OBJECT
public:
   KeySetup(SOptions *opt,QWidget *parent=0,const char *name=0);
protected slots:
   void okPressed();
   void defaultsPressed();
   void keyLeft1();
   void keyRight1();
   void keyAcc1();
   void keyShot1();
   void keyMine1();
   void keyLeft2();
   void keyRight2();
   void keyAcc2();
   void keyShot2();
   void keyMine2();
   void keyStart();
protected:
   virtual void keyPressEvent(QKeyEvent *ev);
   void setButtons(int pl,int b);
private:
   SOptions *options;
   QLabel *keyName[2][PlayerKeyNum],*fKeyName[FunctionKeyNum];
   QPushButton *button[2][PlayerKeyNum],*fbutton[FunctionKeyNum];
   QPushButton *ok, *cancel, *defaults;
   int waitForKey,player,key[2][PlayerKeyNum],fkey[FunctionKeyNum];
};

class ConfigSetup:public QDialog
{
   Q_OBJECT
public:
   ConfigSetup(SConfig *custom,SOptions *opt,QWidget *parent,const char* name=0);

protected slots:
   void okPressed();
   void helpPressed();
   void configSelected(int num);
   void textChanged(const char*);
   void gotFocus();
   void lostFocus();
protected:   
   void valueChanged(int ednum,int val);
   void valueChanged(int ednum,float val);
   void displayConfig(SConfig cfg);
   int validateConfig(bool all);
   void setValue(int ednum,int val);
   void setValue(int ednum,float val);
   void setValue(int ednum,unsigned val);
private:
   
   enum {EditRefreshTime=0,EditRedHitpoints,EditBlueHitpoints,
         EditSunEnergy,EditGravity,
         EditShotSpeed,EditShotEnergyNeed,EditMaxBullets,EditBulletDamage,
         EditMineFuel,EditMineEnergyNeed,EditMineActivateTime,
         EditMineDamage,EditMaxMines,EditGamespeed,
         EditPosX,EditPosY,EditVelX,EditVelY,
         EditAcc,EditEnergyNeed,EditRotationSpeed,
         EditRotationEnergyNeed,EditShipDamage,
         EditNum};
   enum {BoxGraphic=0,BoxHitpoints,BoxConfig,BoxSun,BoxBullet,BoxMine,
         BoxGeneral,BoxStart,BoxShip,BoxNum};
   enum Type {VarInt,VarFloat};
   static char LabelName[EditNum][20];
   static int Parent[EditNum];
   static int Position[EditNum];
   static int BoxLineNum[BoxNum];
   static char BoxName[BoxNum][20];
   static float EditVal[EditNum][3];
   static Type VarType[EditNum];

   QPushButton *ok,*cancel,*help;
   MyLineEdit* edit[EditNum];
   QValidator* validator[EditNum];
   QComboBox *configCombo;
   QLabel *message;
   SConfig *customConfig,config;
   SOptions *gameOptions,options;
};
#endif
